
//Title:        MiniTeller
//Version:
//Copyright:    Copyright (c) 1998
//Author:       Nicholas Riley
//Company:      Brandeis University
//Description:  A small bank implementation

package miniteller;

import java.awt.*;
import com.sun.java.swing.*;
import borland.jbcl.layout.*;
import java.awt.event.*;
import org.omg.CORBA.UserException;

class MenuFrame extends JFrame {
  JLabel title = new JLabel();
  GridBagLayout tellerGBL = new GridBagLayout();
  JPanel opsPanel = new JPanel();
  GridLayout btnLayout = new GridLayout();
  JButton listButton = new JButton();
  JButton openButton = new JButton();
  JButton depButton = new JButton();
  JButton wdButton = new JButton();
  JButton transButton = new JButton();
  JButton exitButton = new JButton();

  protected MiniTeller mt = null;

  MenuFrame() {
    jbInit();
    pack();
    show();
  }

  public MenuFrame(MiniTeller teller) {
    this();
    this.mt = teller;
  }

  private void jbInit() {
    this.getContentPane().setLayout(tellerGBL);
    this.setResizable(false);
    this.setTitle("MiniTeller");
    title.setText("Select a function:");
    title.setFont(new Font("SansSerif", 1, 14));
    listButton.setText("List Accounts");
    listButton.addActionListener(new OpLaunchListener() {
      public Frame opFrame() throws UserException
      { return new AcctListFrame(mt); }
    });
    openButton.setText("Open Account");
    openButton.addActionListener(new OpLaunchListener() {
      protected Frame opFrame() throws UserException
      { return new OpenAcctFrame(mt); }
    });
    depButton.setText("Make Deposit");
    depButton.addActionListener(
      new AcctOpLaunchListener(AcctOpsFrame.OP_DEPOSIT));
    wdButton.setText("Make Withdrawal");
    wdButton.addActionListener(
      new AcctOpLaunchListener(AcctOpsFrame.OP_WITHDRAW));
    transButton.setText("Make Transfer");
    transButton.addActionListener(
      new AcctOpLaunchListener(AcctOpsFrame.OP_TRANSFER));
    exitButton.setText("Exit");
    exitButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        exit();
      }
    });
    btnLayout.setVgap(5);
    btnLayout.setColumns(1);
    btnLayout.setRows(6);
    btnLayout.setHgap(5);
    opsPanel.setLayout(btnLayout);
    this.getContentPane().add(title, new GridBagConstraints2(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
    this.getContentPane().add(opsPanel, new GridBagConstraints2(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    opsPanel.add(listButton, null);
    opsPanel.add(openButton, null);
    opsPanel.add(depButton, null);
    opsPanel.add(wdButton, null);
    opsPanel.add(transButton, null);
    opsPanel.add(exitButton, null);
  }

  void exit() {
    System.exit(0);
  }

  class AcctOpLaunchListener extends OpLaunchListener {
    int op;
    AcctOpLaunchListener(int opCode) {
      this.op = opCode;
    }
    protected Frame opFrame() { return new AcctOpsFrame(mt, op); }
  }

  abstract class OpLaunchListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      Frame f = null;
      try {
      f = opFrame();
      f.addWindowListener(reenabler);
      f.setVisible(true);
      } catch (UserException ue) {
        f.dispose();
        new ErrorDialog(Frame.this, ue);
      }
    }
    protected abstract Frame opFrame() throws UserException;
  }

  ReenableMenu reenabler = new ReenableMenu();

  class ReenableMenu extends WindowAdapter {
    public void windowOpened(WindowEvent e) {
      MenuFrame.this.setEnabled(false);
    }
    public void windowClosing(WindowEvent e) {
      MenuFrame.this.setEnabled(true);
    }
  }
}
