
//Title:        MiniTeller
//Version:      
//Copyright:    Copyright (c) 1998
//Author:       Nicholas Riley
//Company:      Brandeis University
//Description:  A small bank implementation

package miniteller;

import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import borland.jbcl.layout.*;

class OpenAcctFrame extends JFrame implements ActionListener {

  JPanel detailPanel = new JPanel();
  GridBagLayout detailGBL = new GridBagLayout();
  JPanel buttonPane = new JPanel();

  BorderLayout panelsLayout = new BorderLayout();
  JLabel lnLabel = new JLabel("Last name:", SwingConstants.RIGHT);
  JTextField lnField = new JTextField(15);
  JLabel fnLabel = new JLabel("First name:", SwingConstants.RIGHT);
  JTextField fnField = new JTextField(15);
  JLabel addrLabel = new JLabel("Address:", SwingConstants.RIGHT);
  JTextField addr1Field = new JTextField(30);
  JTextField addr2Field = new JTextField(30);
  JLabel phoneLabel = new JLabel("Phone:", SwingConstants.RIGHT);
  JTextField phoneField = new JTextField(15);
  JLabel ssnLabel = new JLabel("SSN:", SwingConstants.RIGHT);
  JTextField ssnField = new JTextField(11);
  JLabel balLabel = new JLabel("Initial balance: $", SwingConstants.RIGHT);
  JTextField balField = new JTextField(9);
  JButton openButton = new JButton("Open");
  JButton cancelButton = new JButton("Cancel");

  protected MiniTeller mt = null;

  public OpenAcctFrame(MiniTeller teller) {
    mt = teller;
    jbInit();
    pack();
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
  }

  private void jbInit() {
    this.setTitle("Open Account");
    this.getContentPane().setLayout(panelsLayout);
    panelsLayout.setVgap(10);
    balField.setHorizontalAlignment(SwingConstants.RIGHT);
    openButton.addActionListener(this);
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });
    detailPanel.setLayout(detailGBL);
    detailPanel.add(lnLabel, new GridBagConstraints2(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 10, 0, 0), 0, 0));
    detailPanel.add(lnField, new GridBagConstraints2(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 10), 0, 0));
    detailPanel.add(fnLabel, new GridBagConstraints2(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 10, 0, 0), 0, 0));
    detailPanel.add(fnField, new GridBagConstraints2(3, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 10), 0, 0));
    detailPanel.add(addrLabel, new GridBagConstraints2(0, 1, 1, 2, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 0, 0, 0), 0, 0));
    detailPanel.add(addr1Field, new GridBagConstraints2(1, 1, 4, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 10), 0, 0));
    detailPanel.add(addr2Field, new GridBagConstraints2(1, 2, 4, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 0, 10), 0, 0));
    detailPanel.add(phoneLabel, new GridBagConstraints2(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 0, 0, 0), 0, 0));
    detailPanel.add(phoneField, new GridBagConstraints2(1, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.SOUTHEAST, GridBagConstraints.BOTH, new Insets(5, 5, 0, 10), 0, 0));
    detailPanel.add(ssnLabel, new GridBagConstraints2(2, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 0, 0, 0), 0, 0));
    detailPanel.add(ssnField, new GridBagConstraints2(3, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 0, 10), 0, 0));
    detailPanel.add(balField, new GridBagConstraints2(2, 4, 2, 2, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.VERTICAL, new Insets(8, 5, 0, 10), 0, 0));
    detailPanel.add(balLabel, new GridBagConstraints2(1, 4, 1, 1, 0.0, 0.0
            ,GridBagConstraints.SOUTHEAST, GridBagConstraints.BOTH, new Insets(10, 10, 0, 0), 0, 0));
    this.getContentPane().add(buttonPane, BorderLayout.SOUTH);
    buttonPane.add(openButton, null);
    buttonPane.add(cancelButton, null);
    this.getContentPane().add(detailPanel, BorderLayout.CENTER);
  }

  Account acc;
  boolean acCreated = false;

  // open account - not all too flexible in layout, but it avoids dumping a
  // huge class/method body in the jbInit() method, creating another anonymous
  // or inner class.
  public void actionPerformed(ActionEvent ev) {
    int acctno;
    String bal;
    try {
    if (!acCreated) {
      float openBal;
      try {
      openBal = Float.valueOf(balField.getText()).floatValue();
			if (openBal == Float.POSITIVE_INFINITY) {
				new ErrorDialog(this, new DataInvalid(
					"Opening balance is too large - please try again."));
				return;
			}
      } catch (NumberFormatException e) {
        new ErrorDialog(this, new DataInvalid(
          "Cannot create account: specified opening balance is not a number " +
          "in the form xxxx[.yy], e.g. 13904.97."));
        return;
      }
      acc = mt.newAccount(openBal);
      acCreated = true;
      cancelButton.setEnabled(false); // can't back out at this point
      setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
      balField.setEditable(false);    // or change balance; it's set
      balField.setBackground(SystemColor.control);
    }
    acctno = acc.acctno();
    bal = acc.getBasicInfo()[Account.bidx_bal];
    acc.setCustomerInfo(fnField.getText(), lnField.getText(),
                        addr1Field.getText(), addr2Field.getText(),
                        phoneField.getText(), ssnField.getText());
    } catch (AccessViolation e) { new ErrorDialog(this, e); return; }
      catch (DataInvalid e) { new ErrorDialog(this, e); return; }
    String[][] mes = {{"New account number:", Integer.toString(acctno)},
                      {"Opening balance:", bal}};
    new NotifyDialog(this, "New account successfully created.", mes);
    close();
  }

  protected void close() {
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    dispatchEvent(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
  }
}
