package miniteller;

class CustInfo {
  private static final int ADDR_MAX = 80, PHONE_MAX = 20, SSN_SIZE = 9;
  protected static long SSN_INVALID = -1;

  String addr1;
  String addr2;
  String phone;
  long ssn = SSN_INVALID;

  private void setStrings(String addr1, String addr2, String phone)
   throws DataInvalid {
    if (addr1 != null && addr1.length() > ADDR_MAX)
      throw new DataInvalid("first line of address", ADDR_MAX);
    if (addr2 != null && addr2.length() > ADDR_MAX)
      throw new DataInvalid("second line of address", ADDR_MAX);
    if (phone != null && phone.length() > PHONE_MAX)
      throw new DataInvalid("phone number", PHONE_MAX);
    this.addr1 = addr1; this.addr2 = addr2; this.phone = phone;
  }

  public CustInfo() throws DataInvalid { // null
    setStrings("[unavailable]", "", "[unavailable]");
  }

  public CustInfo(String addr1, String addr2, String phone, long ssn)
   throws DataInvalid {
    setStrings(addr1, addr2, phone);
    this.ssn = ssn;
  }

  public CustInfo(String addr1, String addr2, String phone, String ssn)
   throws DataInvalid {
    setStrings(addr1, addr2, phone);
    setSSN(ssn);
  }

  protected void setSSN(String ssn) throws DataInvalid {
    StringBuffer sb = new StringBuffer(SSN_SIZE);
    int i = 0, sbl = ssn.length();
    ssn = ssn.trim();
    char[] ssna = new char[sbl];
    ssn.getChars(0, sbl, ssna, 0);
    for (sbl = 0 ; i < ssna.length ; i++) {
      if (ssna[i] < '0' || ssna[i] > '9') continue;
      sb.append(ssna[i]);
      if (++sbl == SSN_SIZE) {
        try { this.ssn = Long.parseLong(sb.toString()); return; }
        catch (NumberFormatException e) {
          throw new DataInvalid(
           "Internal error: Could not parse Social Security Number");
        }
      } // if complete
    } // iterate through ssna[]
    throw new DataInvalid(
      "Database error: cannot accept Social Security number because its " +
      "length is not " + SSN_SIZE + " characters");
  }  

  public String getFormattedSSN() {
    if (ssn == SSN_INVALID) return "[unavailable]";
    StringBuffer sb = new StringBuffer(SSN_SIZE + 2);
    int sbl;
    sb.append(ssn);
    // pad with 0's in the dumbest way possible
    if ( (sbl = SSN_SIZE - sb.length()) != 0)
      for (int i = 0 ; i < sbl ; i++)
      	sb.insert(0, '0');
    // insert hyphens, again in the dumbest way possible
    sb.insert(3, '-');
    sb.insert(6, '-');
    
    return sb.toString();
  }
}
